package KGS::Game::Board;

=head1 NAME

KGS::Game::Board - represents a go board

=head1 SYNOPSIS

   use KGS::Game::Board;

=head1 DESCRIPTION

Please supply a description )

=head2 METHODS

=over 4

=cut

use Games::Go::SimpleBoard;

use base Games::Go::SimpleBoard::;

=item my $board = new $size

Creates a new empty board of the given size. See base class L<Games::Go::SimpleBoard>.

C<< $board->{score}[COLOUR] >> stores the score (if available) for
the given colour, else C<undef>.

C<< $board->{timer}[COLOUR] >> stores the C<< [$time, $count] >> remaining time
info for the given user, if known. C<undef> otherwise.

=cut

sub new {
   my $class = shift;

   #timer    => [],
   #score    => [],
   #last     => COLOUR_...,

   $class->SUPER::new (@_)
}

=item $board->interpret_path ($path)

Interprets the path (as returned by C<KGS::Game::Tree::get_path>) and leaves
the board in the state that it reaches after executing all the pth nodes.

=cut

sub interpret_path {
   my ($self, $path) = @_;

   my $board = $self->{board};

   my @movepath;

   $self->{last}    = COLOUR_WHITE; # black always starts.. ehrm..
   $self->{curnode} = $path->[-1];

   for (@$path) {
      while (my ($k, $v) = each %$_) {
         if ($k =~ /^(\d+),(\d+)$/) {
            push @movepath, [$1, $2, @$v[1,0,2], \$v->[3]];

         } elsif ($k eq "timer") {
            $self->{timer}[0] = $v->[0] if defined $v->[0];
            $self->{timer}[1] = $v->[1] if defined $v->[1];

         } elsif ($k eq "pass") {
            push @movepath, [];

         } elsif ($k eq "score") {
            $self->{score} = $v;
         }
      }
   }

   $self->update (\@movepath);
}

=item $board->is_valid_move ($colour, $x, $y[, $may_suicide])

Returns true if the move of the given colour on the given coordinates is
valid or not.

=cut

sub is_valid_move {
   my ($self, $colour, $x, $y, $may_suicide) = @_;

   my $board = $self->{board};

   return if $board->[$x][$y] & (MARK_B | MARK_W | MARK_KO);

   if ($may_suicide) {
      return 1;
   } else {
      my ($own, $opp) = $colour == COLOUR_BLACK
                           ? (MARK_B, MARK_W)
                           : (MARK_W, MARK_B);

      # try the move
      local $board->[$x][$y] = $board->[$x][$y] | $own;

      return 1 if $x > 0            && $board->[$x-1][$y] & $opp && $self->capture ($opp, $x-1, $y, 1);
      return 1 if $x < $self->{max} && $board->[$x+1][$y] & $opp && $self->capture ($opp, $x+1, $y, 1);
      return 1 if $y > 0            && $board->[$x][$y-1] & $opp && $self->capture ($opp, $x, $y-1, 1);
      return 1 if $y < $self->{max} && $board->[$x][$y+1] & $opp && $self->capture ($opp, $x, $y+1, 1);

      return !$self->capture ($own, $x, $y, 1);
   }
}

1;

=back

=head2 AUTHOR

Marc Lehmann <schmorp@schmorp.de>

=head2 SEE ALSO

L<KGS::Protocol>, L<KGS::Game::Tree>, L<Gtk2::GoBoard>.

=cut

